#!/usr/bin/perl
#   
# -   HTML   KOI8-R
# -      .html   .htm, .shtml, .php  ..
# -  DOCTYPE
# -    .

my $cfg_dir_path="/home/opennet/arc/MAN/1";
my $cfg_src_charset="utf8"; # cp1251|utf8
my $cfg_header_begin_mask=qq~<html xmlns~; #"<body";
my $cfg_header_end_mask=qq~<div class="wiki">~;
my $cfg_footer_begin_mask=qq~<div class="share">~;# "<!-- footer";
my $cfg_footer_end_mask=qq~</html>~;


use strict;
use File::Find;

if (defined $ARGV[0] && -f $ARGV[0]){
    print "Force parse $ARGV[0]\n";
    find(\&process_file, $ARGV[0]);
    exit;
}

if (! -d $cfg_dir_path ){
    die "Dir not found: $cfg_dir_path"; 
}

find(\&process_file, $cfg_dir_path);
exit;


sub process_file{
    my $file_name = $_;
    my $full_path = $File::Find::name;
    my $full_dir = $File::Find::dir;
    if ( -f $full_path && $file_name =~ /^([^\.]+)\.(html|htm|php|shtml|phtml|xhtml)$/){
	my $file_base=$1;
	print "Parsing $file_base...\n";
	
	my $skip_h_flag=0;
	my $skip_f_flag=0;
		
	my $ext_rename_flag=0;
	if ($file_name =~ /\.(htm|php|shtml|phtml|xhtml)$/){
	    $ext_rename_flag=1;
	}
	open(SRC_FILE, "<$full_path");
	flock(SRC_FILE, 1);
	open(DST_FILE, ">$full_path.tmp");
	flock(DST_FILE, 1);
	my $line_cnt=0;
	while(<SRC_FILE>){
	    my $line = $_;
	    $line_cnt++;
	    $line =~ s/\r//g;
	    if ($line_cnt < 3 && $line =~ /^\s*<!DOCTYPE[^>]+>\s*$/){
		next;
	    }

	    if ($skip_h_flag == 1){
		if ($line =~ /$cfg_header_end_mask/){
		    $skip_h_flag = 2;
		}
		next;
	    } elsif ($skip_h_flag == 0 && $cfg_header_begin_mask ne "" && $line =~ /$cfg_header_begin_mask/){
		$skip_h_flag = 1;
	    }

	    if ($skip_f_flag == 1){
		if ($line =~ /$cfg_footer_end_mask/){
		    $skip_f_flag = 2;
		} else {
		    next;
		}
	    } elsif ($skip_f_flag == 0 && $cfg_footer_begin_mask ne "" && $line =~ /$cfg_footer_begin_mask/){
		$skip_f_flag = 1;
		next;
	    }

	    
	    if ($line_cnt < 20 && $line =~ /<[mM][eE][tT][aA]/){
		$line =~ s/\<meta [^\>]+charset[^\>]+\>//i;
	    }
	    $line =~ s/href="\/main\/ru\/([^\"]+)"/href="\/man.shtml?topic=$1"/ig;

	    if ($ext_rename_flag == 1){
		$line =~ s/"([^\"\:]+)\.(htm|php|shtml|phtml|xhtml)"/"$1.html"/g;
		$line =~ s/\=([^\>\:]+)\.(htm|php|shtml|phtml|xhtml)\>/=$1.html>/g;
		$line =~ s/"([^\"\:]+)\.(htm|php|shtml|phtml|xhtml)\#([^"]+)"/"$1.html#$3"/g;
		$line =~ s/\=([^\>\:]+)\.(htm|php|shtml|phtml|xhtml)\#([^>]+)\>/=$1.html#$3>/g;
	    }


	    if ($cfg_src_charset eq "cp1251"){
		print DST_FILE wintokoi($line);
	    } elsif ($cfg_src_charset eq "utf8"){
		print DST_FILE utf8tokoi($line);
	    } else {
		print DST_FILE $line;
	    }
	}
	close(DST_FILE);
	close(SRC_FILE);
	if ($skip_h_flag == 1){
	    print "ERROR: not closed header !\n";
	}elsif ($skip_f_flag == 1){
	    print "ERROR: not closed footer !\n";
	} else {
	    unlink($full_path);
	    rename("$full_path.tmp", "$full_dir/$file_base.html");
	}
	
    }
}


sub wintokoi {
    my $pvdcoderwin=shift;
    $pvdcoderwin=~ tr/\xB8\xC0\xC1\xC2\xC3\xC4\xC5\xC6\xC7\xC8\xC9\xCA\xCB\xCC\xCD\xCE\xCF\xD0\xD1\xD2\xD3\xD4\xD5\xD6\xD7\xD8\xD9\xDA\xDB\xDC\xDD\xDE\xDF\xE0\xE1\xE2\xE3\xE4\xE5\xE6\xE7\xE8\xE9\xEA\xEB\xEC\xED\xEE\xEF\xF0\xF1\xF2\xF3\xF4\xF5\xF6\xF7\xF8\xF9\xFA\xFB\xFC\xFD\xFE\xFF\x91\x92\x93\x94\x96\x97\x9B\x8B\xBB\xAB\xB9\xAD\xA7\xA0\x82\x84\x85/\\xE1\xE2\xF7\xE7\xE4\xE5\xF6\xFA\xE9\xEA\xEB\xEC\xED\xEE\xEF\xF0\xF2\xF3\xF4\xF5\xE6\xE8\xE3\xFE\xFB\xFD\xFF\xF9\xF8\xFC\xE0\xF1\xC1\xC2\xD7\xC7\xC4\xC5\xD6\xDA\xC9\xCA\xCB\xCC\xCD\xCE\xCF\xD0\xD2\xD3\xD4\xD5\xC6\xC8\xC3\xDE\xDB\xDD\xDF\xD9\xD8\xDC\xC0\xD1\'\'\"\"\-\-\>\<\"\"N\-\ \ \,\,\./;
    return $pvdcoderwin;
}

sub utf8tokoi {
    my $pvdcoderwin=shift;
    use Unicode::Map8;
    use Unicode::String qw(utf8);
    my $koi8 = Unicode::Map8->new("koi8-r");
    return $koi8->to8(utf8($pvdcoderwin)->utf16);
;
}

